
test_dir = artifact"testdata"
DJ2000 = 2451545

@testset "SPK Type 13" verbose=true begin 

    # Each kernel contains segments both with and without epoch directories. 
    # Different descriptors are tested to include epoch directories and even/odd windows
    kernels = [joinpath(test_dir, "example1spk_seg13.bsp"), 
               joinpath(test_dir, "example2spk_seg13.bsp")]
    
    # Desired descriptors 
    descidj = [(1, 2), (3, 4)]

    for (ik, kernel) in enumerate(kernels)

        ephj = EphemerisProvider(kernel);
        ephc = CalcephProvider(kernel);
        furnsh(kernel)

        for (cdj, idj) in enumerate(descidj[ik])
            desc = ephj.files[1].desc[idj]

            t1j, t2j = desc.tstart, desc.tend

            cid = Int(desc.cid)
            tid = Int(desc.tid)

            # Test values 
            yc1 = zeros(3);
            yc2 = zeros(6);
            yc3 = zeros(9);
            yc4 = zeros(12);

            ep = t1j:1:t2j
            for j in 1:2000
                
                tj = j == 1 ? t1j : (j == 2 ? t2j : rand(ep))
                tc = tj/86400

                yj1 = ephem_vector3(ephj, cid, tid, tj);
                yj2 = ephem_vector6(ephj, cid, tid, tj);
                yj3 = ephem_vector9(ephj, cid, tid, tj);
                yj4 = ephem_vector12(ephj, cid, tid, tj);
            
                jEphem.ephem_compute!(yc1, ephc, DJ2000, tc, tid, cid, 0);
                jEphem.ephem_compute!(yc2, ephc, DJ2000, tc, tid, cid, 1);
                jEphem.ephem_compute!(yc3, ephc, DJ2000, tc, tid, cid, 2);
                jEphem.ephem_compute!(yc4, ephc, DJ2000, tc, tid, cid, 3);
                        
                ys1 = spkpos("$tid", tj, "J2000", "NONE", "$cid")[1]
                ys2 = spkezr("$tid", tj, "J2000", "NONE", "$cid")[1]

                # Test against CALCEPH
                @test yj1 ≈ yc1 atol=1e-9 rtol=1e-9
                @test yj2 ≈ yc2 atol=1e-9 rtol=1e-9
                @test yj3 ≈ yc3 atol=1e-9 rtol=1e-9
                @test yj4 ≈ yc4 atol=1e-9 rtol=1e-9

                # Test against SPICE
                @test yj1 ≈ ys1 atol=1e-9 rtol=1e-14
                @test yj2 ≈ ys2 atol=1e-9 rtol=1e-14

                if ik == 2 && cdj == 2
                    jatol, jrtol = 1e-6, 1e-9 
                else 
                    jatol, jrtol = 1e-9, 1e-13
                end

                # Test if AUTODIFF works 
                # Position
                @test D¹(t->ephem_vector3(ephj, cid, tid, t), tj) ≈ yj4[4:6] atol=1e-9 rtol=1e-13
                @test D²(t->ephem_vector3(ephj, cid, tid, t), tj) ≈ yj4[7:9] atol=1e-9 rtol=1e-13

                @test D³(t->ephem_vector3(ephj, cid, tid, t), tj) ≈ yj4[10:12] atol=jatol rtol=jrtol

                # Velocity 
                @test D¹(t->ephem_vector6(ephj, cid, tid, t), tj) ≈ yj4[4:9] atol=1e-9 rtol=1e-13
                @test D²(t->ephem_vector6(ephj, cid, tid, t), tj) ≈ yj4[7:12]  atol=jatol rtol=jrtol

                # Acceleration 
                @test D¹(t->ephem_vector9(ephj, cid, tid, t), tj) ≈ yj4[4:12]  atol=jatol rtol=jrtol

            end
        end

        kclear()
    end

end;


